﻿package  
{
	import flash.geom.Matrix;
	import flash.utils.getTimer;
	import flash.display.*;
	import flash.events.*;
	import flash.ui.Keyboard;
	import org.papervision3d.core.clipping.FrustumClipping;
	import org.papervision3d.materials.special.ParticleMaterial;
	import org.papervision3d.objects.special.ParticleField;

	import org.papervision3d.view.Viewport3D;
	import org.papervision3d.core.geom.*;
	import org.papervision3d.core.math.*;
	import org.papervision3d.core.proto.*;
	import org.papervision3d.cameras.*;
	import org.papervision3d.render.*;
	import org.papervision3d.events.*;
	import org.papervision3d.materials.utils.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.materials.shadematerials.*;
	import org.papervision3d.objects.primitives.*;
	import org.papervision3d.objects.parsers.*;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.scenes.*;
	import org.papervision3d.objects.parsers.Collada;
	import org.papervision3d.view.stats.StatsView;
	import org.papervision3d.view.layer.ViewportLayer;
	import org.papervision3d.view.layer.util.ViewportLayerSortMode;

	import jiglib.math.*;
	import jiglib.geometry.JPlane;
	import jiglib.physics.*;
	import jiglib.vehicles.JCar;
	import jiglib.vehicles.JWheel;

	public class CarDrive extends Sprite
	{
		private var viewport:Viewport3D;
		private var scene:Scene3D;
		private var springCamera:SpringCamera3D = new SpringCamera3D();
		private var renderer:BasicRenderEngine;
		private var mylight:PointLight3D;
			
		[Embed(source="/res/wheel.jpg")]
		private var wheelBitmapAsset:Class;
		
		[Embed(source="/res/body.jpg")]
		private var bodyBitmapAsset:Class;
		
		private var carBody:JCar;
		private var carSkin:Collada;
		private var steerFR :DisplayObject3D;
		private var steerFL :DisplayObject3D;
		private var wheelFR :DisplayObject3D;
		private var wheelFL :DisplayObject3D;
		private var wheelRR :DisplayObject3D;
		private var wheelRL :DisplayObject3D;
		
		private var cameraTarget:DisplayObject3D;
		
		public function CarDrive() 
		{
			stage.addEventListener( KeyboardEvent.KEY_DOWN, keyDownHandler );
			stage.addEventListener( KeyboardEvent.KEY_UP, keyUpHandler );
			stage.quality = StageQuality.MEDIUM;
			init3D();
		}
		
		private function init3D():void
		{
			viewport = new Viewport3D(800, 600);
			viewport.containerSprite.sortMode = ViewportLayerSortMode.INDEX_SORT;
			scene = new Scene3D();
			this.addChild(viewport);
			
			mylight = new PointLight3D(true, true);
			mylight.x = 0;
			mylight.y = 600;
			mylight.z = -300;
			 
			var planeSkin:Plane = new Plane(new WireframeMaterial(0x3366CC), 800, 800, 4, 4 );
			viewport.getChildLayer(planeSkin).layerIndex = 0;
			scene.addChild(planeSkin);
			var plane:JPlane = new JPlane(planeSkin);
			plane.MoveTo(new JNumber3D(0, 260, 0), JMatrix3D.multiply(JMatrix3D.rotationY(Math.PI / 2), JMatrix3D.rotationX(Math.PI / 2)));
			PhysicsSystem.getInstance().AddBody(plane);
			 
			//init car skin
			var wheelBitmap:Bitmap = new wheelBitmapAsset() as Bitmap;
			var bodyBitmap:Bitmap = new bodyBitmapAsset() as Bitmap;
			var materiaList :MaterialsList = new MaterialsList();
			materiaList.addMaterial(new BitmapMaterial(bodyBitmap.bitmapData), "materialBody");
			materiaList.addMaterial(new BitmapMaterial(wheelBitmap.bitmapData), "materialWheel");
			carSkin = new Collada("res/car.xml", materiaList, 0.3);
			carSkin.addEventListener(FileLoadEvent.LOAD_COMPLETE,onCarLoaded);
			 
			cameraTarget = new DisplayObject3D();
			
			springCamera.target = cameraTarget
			springCamera.mass = 10;
			springCamera.damping = 10;
			springCamera.stiffness = 1;
			springCamera.positionOffset = new Number3D(0, 200, -200);

			renderer = new BasicRenderEngine();
			
			
			var stats:StatsView = new StatsView(renderer);
			addChild(stats);
			
			// ついでにパーティクル
			var particles:ParticleField = new ParticleField(new ParticleMaterial(0xFFFFFF, 1), 400, 4, 2000, 500, 2000);
			particles.y = 500;
            scene.addChild( particles );
			
			// ついでに背景の作成 不要なら以下4行を削除ください
			var bgMatrix:Matrix = new Matrix();
			bgMatrix.rotate(90 * Math.PI / 180);
			graphics.beginGradientFill("linear", [0x555555, 0x001122], [100, 100], [0, 255], bgMatrix);
			graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
		}
		
		private function onCarLoaded(e:FileLoadEvent = null):void
		{
			scene.addChild(carSkin);
			 
			//init car physics
			carBody = new JCar(carSkin);
			carBody.Chassis.MoveTo(new JNumber3D( 0, 310, -100), JMatrix3D.rotationY(80));
			carBody.Chassis.setMass(5);
			carBody.Chassis.Material.DynamicFriction = 0.3;
			carBody.Chassis.SideLengths = new JNumber3D(50, 20, 70);
			PhysicsSystem.getInstance().AddBody(carBody.Chassis);
			 
			carBody.SetupWheel("wheel_fl", new JNumber3D( -30, -5, 32), 2, 2, 0.5, 10, 0.7, 0.6, 1);
			carBody.SetupWheel("wheel_fr", new JNumber3D(30, -5, 32), 2, 2, 0.5, 10, 0.7, 0.6, 1);
			carBody.SetupWheel("wheel_bl", new JNumber3D( -30, -5, -32), 2, 2, 0.5, 10, 0.7, 0.6, 1);
			carBody.SetupWheel("wheel_br", new JNumber3D(30, -5, -32), 2, 2, 0.5, 10, 0.7, 0.6, 1);
			 
			steerFR = carSkin.getChildByName( "Steer_FR", true );
			steerFL = carSkin.getChildByName( "Steer_FL", true );
			wheelFR = steerFR.getChildByName( "Steer_FR_PIVOT", true );
			wheelFL = steerFL.getChildByName( "Steer_FL_PIVOT", true );
			wheelRR = carSkin.getChildByName( "Wheel_RR", true );
			wheelRL = carSkin.getChildByName( "Wheel_RL", true );
			
			var vplCar:ViewportLayer = viewport.getChildLayer(carSkin);
			vplCar.addDisplayObject3D(steerFR);
			vplCar.addDisplayObject3D(steerFL);
			vplCar.addDisplayObject3D(wheelRR);
			vplCar.addDisplayObject3D(wheelRL);
			vplCar.layerIndex =1;
			 
			this.addEventListener(Event.ENTER_FRAME, loop3D);
		}
		
		private function keyDownHandler(event :KeyboardEvent):void
		{
			switch(event.keyCode)
			{
				case Keyboard.UP:
					carBody.SetAccelerate(1);
					break;
				case Keyboard.DOWN:
					carBody.SetAccelerate(-1);
					break;
				case Keyboard.LEFT:
					carBody.SetSteer(["wheel_fl", "wheel_fr"], -1);
					break;
				case Keyboard.RIGHT:
					carBody.SetSteer(["wheel_fl", "wheel_fr"], 1);
					break;
				case Keyboard.SPACE:
					carBody.SetHBrake(1);
					break;
			}
		}


		private function keyUpHandler(event:KeyboardEvent):void
		{
			switch(event.keyCode)
			{
				case Keyboard.UP:
					carBody.SetAccelerate(0);
					break;
					
				case Keyboard.DOWN:
					carBody.SetAccelerate(0);
					break;
					
				case Keyboard.LEFT:
					carBody.SetSteer(["wheel_fl", "wheel_fr"], 0);
					break;
					
				case Keyboard.RIGHT:
					carBody.SetSteer(["wheel_fl", "wheel_fr"], 0);
					break;
				case Keyboard.SPACE:
				   carBody.SetHBrake(0);
			}
		}
		 
		private function updateWheelSkin():void
		{
			steerFL.rotationX = carBody.Wheels["wheel_fl"].GetSteerAngle();
			steerFR.rotationX = carBody.Wheels["wheel_fr"].GetSteerAngle();
			
			wheelFR.roll(carBody.Wheels["wheel_fr"].getRollAngle());
			wheelFL.roll(-carBody.Wheels["wheel_fl"].getRollAngle());
			wheelRR.roll(carBody.Wheels["wheel_br"].getRollAngle());
			wheelRL.roll(-carBody.Wheels["wheel_bl"].getRollAngle());
		}
		 
		private function loop3D( event :Event ):void
		{
			
			PhysicsSystem.getInstance().Integrate(0.1);
			
			updateWheelSkin();
			
			cameraTarget.copyTransform(carSkin);
			
			renderer.renderScene( scene, springCamera, viewport );
		}
	}
	
}